# -*- coding: utf-8 -*-

# Copyright :copyright: 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import time
from socket import error as SocketError
import errno

import pyactiveresource
from pyactiveresource.connection import (
    BadRequest,
    UnauthorizedAccess,
    ForbiddenAccess,
    ResourceNotFound,
    MethodNotAllowed,
    ResourceConflict,
    ResourceInvalid,
    ClientError,
    ServerError,
    ConnectionError
)
from shopify import ShopifyResource, Limits
from shopify.base import ShopifyConnection

from wrapt import apply_patch

from shopify_python_api_patcher import max_retries, logger

def _open(self, *args, **kwargs):
    retries = max_retries
    self.response = None
    while retries > 0:
        retries -= 1
        try:
            self.response = super(ShopifyConnection, self)._open(*args, **kwargs)
            break
        except ServerError as e:
            if retries <= 0:
                raise

            time.sleep(0.5)
        except (BadRequest, UnauthorizedAccess, ForbiddenAccess, ResourceNotFound, MethodNotAllowed, ResourceConflict, ResourceInvalid) as e:
            self.response = e.response
            raise
        except ClientError as e:
            logger.exception(e)

            if retries <= 0:
                raise

            self.response = e.response
            if e.code in [429, 430]:
                try:
                    no_credit = Limits.credit_maxed()
                except:
                    no_credit = False
                wait_time = 30 if no_credit else 0.5
                time.sleep(wait_time)
            elif e.code in [104]:
                ShopifyResource.set_site(ShopifyResource.get_site())
                time.sleep(3)
            else:
                raise
        except (ConnectionError, pyactiveresource.formats.Error) as e:
            logger.exception(e)

            if retries <= 0:
                if hasattr(e, 'response'):
                    self.response = e.response
                raise

            ShopifyResource.set_site(ShopifyResource.get_site())
        except SocketError as e:
            logger.exception(e)

            if retries <= 0 or e.errno != errno.ECONNRESET:
                raise

            time.sleep(30)
            ShopifyResource.set_site(ShopifyResource.get_site())

    return self.response

apply_patch(ShopifyConnection, '_open', _open)
