# -*- coding: utf-8 -*-

# Copyright :copyright: 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import sys
from socket import error as SocketError

import pyactiveresource
from pyactiveresource.connection import (
    BadRequest,
    UnauthorizedAccess,
    ForbiddenAccess,
    ResourceNotFound,
    MethodNotAllowed,
    ResourceConflict,
    ResourceInvalid,
    ClientError,
    ServerError,
    ConnectionError,
    Error
)

import shopify_python_api_patcher
import shopify

from mock import MagicMock
import pytest

shopify_python_api_patcher.max_retries = 2
shopify.ShopifyResource.set_site('http://localhost')

def test_open():
    unprocessable_error = [
        'BadRequest', 'UnauthorizedAccess', 'ForbiddenAccess', 'ResourceNotFound',
        'MethodNotAllowed', 'ResourceConflict', 'ResourceInvalid'
    ]
    for cls_name in unprocessable_error:
        fake_open = MagicMock()
        class_obj = getattr(sys.modules[__name__], cls_name)
        fake_open.side_effect = class_obj()

        pyactiveresource.connection.Connection._open = fake_open

        with pytest.raises(class_obj):
            shopify.Shop.current()

        fake_open.assert_called_once()

    # Test ServerError processing
    fake_open = MagicMock()
    fake_open.side_effect = ServerError()
    pyactiveresource.connection.Connection._open = fake_open

    with pytest.raises(ServerError):
        shopify.Shop.current()

    fake_open.call_count = 2

    # Test ClientError with code 429 processing
    fake_open = MagicMock()
    client_error = ClientError()
    client_error.code = 429
    fake_open.side_effect = client_error
    pyactiveresource.connection.Connection._open = fake_open

    with pytest.raises(ClientError):
        shopify.Shop.current()

    fake_open.call_count = 2

    # Test ClientError with code 301 processing
    fake_open = MagicMock()
    client_error = ClientError()
    client_error.code = 301
    fake_open.side_effect = client_error
    pyactiveresource.connection.Connection._open = fake_open

    with pytest.raises(ClientError):
        shopify.Shop.current()

    fake_open.call_count = 1

    # Test formats Error processing
    fake_open = MagicMock()
    fake_open.side_effect = pyactiveresource.formats.Error()
    pyactiveresource.connection.Connection._open = fake_open

    with pytest.raises(pyactiveresource.formats.Error):
        shopify.Shop.current()

    fake_open.call_count = 2

    # Test SocketError processing
    fake_open = MagicMock()
    fake_open.side_effect = SocketError()
    pyactiveresource.connection.Connection._open = fake_open

    with pytest.raises(SocketError):
        shopify.Shop.current()

    fake_open.call_count = 1
